@@ -401,6 +401,7 @@ static int count_delete_lines (FILE *fp, BODY *b, long *length, size_t datelen)
  	M_CM_DISPLAY	displaying output to the user
 	M_CM_UPDATE	update structures in memory after syncing
 	M_CM_DECODE_PGP	used for decoding PGP messages
+        M_CM_CHARCONV	perform character set conversion
    chflags	flags to mutt_copy_header()
  */
 
@@ -495,6 +496,8 @@ _mutt_copy_message (FILE *fpout, FILE *fpin, HEADER *hdr, BODY *body,
       s.flags |= M_DISPLAY;
     if (flags & M_CM_WEED)
       s.flags |= M_WEED;
+    if (flags & M_CM_CHARCONV)
+      s.flags |= M_CHARCONV;
 
 
     mutt_body_handler (body, &s);
@@ -562,8 +565,10 @@ _mutt_append_message (CONTEXT *dest, FILE *fpin, CONTEXT *src, HEADER *hdr,
     chflags |= CH_FROM;
   chflags |= (dest->magic == M_MAILDIR ? CH_NOSTATUS : CH_UPDATE);
   r = _mutt_copy_message (msg->fp, fpin, hdr, body, flags, chflags);
-  if (mx_close_message (&msg) != 0)
-    return (-1);
+  if (mx_commit_message (msg, dest) != 0)
+    r = -1;
+
+  mx_close_message (&msg);
   return r;
 }
 
