@@ -394,36 +394,6 @@ static void imap_quote_string (char *dest, size_t slen, const char *src)
   *pt = 0;
 }
 
-static void imap_unquote_string (char *s)
-{
-  char *d = s;
-
-  if (*s == '\"')
-    s++;
-  else
-    return;
-
-  while (*s)
-  {
-    if (*s == '\"')
-    {
-      *d = '\0';
-      return;
-    }
-    if (*s == '\\')
-    {
-      s++;
-    }
-    if (*s)
-    {
-      *d = *s;
-      d++;
-      s++;
-    }
-  }
-  *d = '\0';
-}
-
 static int imap_read_bytes (FILE *fp, CONNECTION *conn, long bytes)
 {
   long pos;
@@ -1732,6 +1702,7 @@ int imap_append_message (CONTEXT *ctx, MESSAGE *msg)
   int port;
   size_t len;
   int c, last;
+  CONNECTION *conn = CTX_DATA->conn;
 
   if (imap_parse_path (ctx->path, host, sizeof (host), &port, &pc))
     return (-1);
@@ -1752,18 +1723,25 @@ int imap_append_message (CONTEXT *ctx, MESSAGE *msg)
     len++;
   }
   rewind(fp);
-  
-  mutt_message _("Sending APPEND command ...");
 
   imap_quote_string (mbox, sizeof (mbox), mailbox);
+
+  /* hack from Nathan Neulinger */
+  mutt_message _("Sending CREATE command ...");
+  imap_make_sequence (seq, sizeof (seq));
+  snprintf (buf, sizeof (buf), "%s CREATE %s\r\n", seq, mbox);
+  imap_exec (buf, sizeof (buf), CONN_DATA, seq, buf, IMAP_OK_FAIL);
+
+  mutt_message _("Sending APPEND command ...");
+
   imap_make_sequence (seq, sizeof (seq));
   snprintf (buf, sizeof (buf), "%s APPEND %s {%d}\r\n", seq, mbox, len);
 
-  mutt_socket_write (CTX_DATA->conn, buf);
+  mutt_socket_write (conn, buf);
 
   do 
   {
-    if (mutt_socket_read_line_d (buf, sizeof (buf), CTX_DATA->conn) < 0)
+    if (mutt_socket_read_line_d (buf, sizeof (buf), conn) < 0)
     {
       fclose (fp);
       return (-1);
@@ -1802,19 +1780,19 @@ int imap_append_message (CONTEXT *ctx, MESSAGE *msg)
     buf[len++] = c;
 
     if(len > sizeof(buf) - 3)
-      flush_buffer(buf, &len, CTX_DATA->conn);
+      flush_buffer(buf, &len, conn);
   }
   
   if(len)
-    flush_buffer(buf, &len, CTX_DATA->conn);
+    flush_buffer(buf, &len, conn);
 
     
-  mutt_socket_write (CTX_DATA->conn, "\r\n");
+  mutt_socket_write (conn, "\r\n");
   fclose (fp);
 
   do
   {
-    if (mutt_socket_read_line_d (buf, sizeof (buf), CTX_DATA->conn) < 0)
+    if (mutt_socket_read_line_d (buf, sizeof (buf), conn) < 0)
       return (-1);
 
     if (buf[0] == '*' && imap_handle_untagged (CTX_DATA, buf) != 0)
@@ -2019,7 +1997,6 @@ int imap_check_mailbox (CONTEXT *ctx, int *index_hint)
   char seq[8];
   char buf[LONG_STRING];
   static time_t checktime=0;
-  int msgcount = ctx->msgcount;
 
   if (ImapCheckTime)
   {
